@@ -67,7 +67,7 @@ func (e *DefaultRTREventHandler) RequestCache(c *Client) {
 		} else {
 			c.SendROAs(sessionId, serial, roas)
 			if e.Log != nil {
-				e.Log.Debugf("%v < Sent ROAs (current serial %v)", c, serial)
+				e.Log.Debugf("%v < Sent ROAs (current serial %d, session: %d)", c, serial, sessionId)
 			}
 		}
 	}
@@ -93,7 +93,7 @@ func (e *DefaultRTREventHandler) RequestNewVersion(c *Client, sessionId uint16,
 		} else {
 			c.SendROAs(sessionId, serial, roas)
 			if e.Log != nil {
-				e.Log.Debugf("%v < Sent ROAs (current serial %v)", c, serial)
+				e.Log.Debugf("%v < Sent ROAs (current serial %d, session from client: %d)", c, serial, sessionId)
 			}
 		}
 	}
@@ -126,7 +126,8 @@ type Server struct {
 	pduRetryInterval   uint32
 	pduExpireInterval  uint32
 
-	log Logger
+	log        Logger
+	logverbose bool
 }
 
 type ServerConfiguration struct {
@@ -141,7 +142,8 @@ type ServerConfiguration struct {
 	RetryInterval   uint32
 	ExpireInterval  uint32
 
-	Log Logger
+	Log        Logger
+	LogVerbose bool
 }
 
 func NewServer(configuration ServerConfiguration, handler RTRServerEventHandler, simpleHandler RTREventHandler) *Server {
@@ -186,7 +188,8 @@ func NewServer(configuration ServerConfiguration, handler RTRServerEventHandler,
 		pduRetryInterval:   retryInterval,
 		pduExpireInterval:  expireInterval,
 
-		log: configuration.Log,
+		log:        configuration.Log,
+		logverbose: configuration.LogVerbose,
 	}
 }
 
@@ -347,8 +350,10 @@ func (s *Server) AddROAs(roas []ROA) {
 	roaCurrent := s.roaCurrent
 
 	added, removed, unchanged := ComputeDiff(roas, roaCurrent)
-	if s.log != nil {
+	if s.log != nil && s.logverbose {
 		s.log.Debugf("Computed diff: added (%v), removed (%v), unchanged (%v)", added, removed, unchanged)
+	} else if s.log != nil {
+		s.log.Debugf("Computed diff: added (%d), removed (%d), unchanged (%d)", len(added), len(removed), len(unchanged))
 	}
 	curDiff = append(added, removed...)
 	s.roalock.RUnlock()
@@ -501,8 +506,7 @@ func (s *Server) Start(bind string) error {
 	if err != nil {
 		return err
 	}
-	s.loopTCP(tcplist, "tcp", s.acceptClientTCP)
-	return nil
+	return s.loopTCP(tcplist, "tcp", s.acceptClientTCP)
 }
 
 func (s *Server) acceptClientTCP(tcpconn net.Conn) error {
@@ -583,27 +587,34 @@ func (s *Server) acceptClientSSH(tcpconn net.Conn) error {
 
 type ClientCallback func(net.Conn) error
 
-func (s *Server) loopTCP(tcplist net.Listener, logEnv string, clientCallback ClientCallback) {
+func (s *Server) loopTCP(tcplist net.Listener, logEnv string, clientCallback ClientCallback) error {
 	for {
-		tcpconn, _ := tcplist.Accept()
+		tcpconn, err := tcplist.Accept()
+		if err != nil {
+			if s.log != nil {
+				s.log.Errorf("Failed to accept %s connection: %s", logEnv, err)
+			}
+			continue
+		}
 
 		if s.maxconn > 0 && s.connected >= s.maxconn {
 			if s.log != nil {
-				s.log.Warnf("Could not accept %v connection from %v (not enough slots avaible: %v)", logEnv, tcpconn.RemoteAddr(), s.maxconn)
+				s.log.Warnf("Could not accept %s connection from %v (not enough slots available: %d)", logEnv, tcpconn.RemoteAddr(), s.maxconn)
 			}
 			tcpconn.Close()
 		} else {
 			if s.log != nil {
-				s.log.Infof("Accepted %v connection from %v (%v/%v)", logEnv, tcpconn.RemoteAddr(), s.connected+1, s.maxconn)
+				s.log.Infof("Accepted %s connection from %v (%d/%d)", logEnv, tcpconn.RemoteAddr(), s.connected+1, s.maxconn)
 			}
 			if clientCallback != nil {
 				err := clientCallback(tcpconn)
 				if err != nil && s.log != nil {
-					s.log.Errorf("Error with %v client %v: %v", logEnv, tcpconn.RemoteAddr(), err)
+					s.log.Errorf("Error with %s client %v: %v", logEnv, tcpconn.RemoteAddr(), err)
 				}
 			}
 		}
 	}
+	return nil
 }
 
 func (s *Server) StartSSH(bind string, config *ssh.ServerConfig) error {
@@ -612,8 +623,7 @@ func (s *Server) StartSSH(bind string, config *ssh.ServerConfig) error {
 		return err
 	}
 	s.sshconfig = config
-	s.loopTCP(tcplist, "ssh", s.acceptClientSSH)
-	return nil
+	return s.loopTCP(tcplist, "ssh", s.acceptClientSSH)
 }
 
 func (s *Server) StartTLS(bind string, config *tls.Config) error {
@@ -621,8 +631,7 @@ func (s *Server) StartTLS(bind string, config *tls.Config) error {
 	if err != nil {
 		return err
 	}
-	s.loopTCP(tcplist, "tls", s.acceptClientTCP)
-	return nil
+	return s.loopTCP(tcplist, "tls", s.acceptClientTCP)
 }
 
 func (s *Server) GetClientList() []*Client {
